//
//  dumper.m
//  PLOP-Demo
//
//  Created by Rainer Schaaf on 20.12.19.
//  Copyright © 2019 PDFlib GmbH. All rights reserved.
//

#import "dumper.h"
#import "PLOP_ios/PLOP_objc.h"
#import "UIKit/UIKit.h"

@implementation dumper

+ (NSString *) run: (NSString *) fileName error:(NSError**)error {
    NSMutableString *buffer = [[NSMutableString alloc]init];

    // This is where input files live. Adjust as necessary.
    NSString *searchpath = [[NSBundle mainBundle] resourcePath];
    
    /* global option list */
    NSString *globaloptlist = [NSString stringWithFormat:@"searchpath={{%@}}", searchpath];
    
    PLOP *plop = [[PLOP alloc] init];
    if (!plop) {
        *error = [[NSError alloc] initWithDomain:@"PLOP out of memory error." code:[plop get_errnum] userInfo:nil];
        return nil;
    }
    
    @try
    {
        NSInteger count, pcosmode, plainmetadata, recipientcount;
        NSInteger objtype, i, doc;
        NSString *docoptlist = @"requiredmode=minimum";
        
        [plop set_option:globaloptlist];
        
        if ((doc = [plop open_document:fileName optlist:docoptlist]) == -1)
        {
            NSString *errorText = [NSString stringWithFormat:@"Error %ld in %@(): %@\n",
                                   (long)[plop get_errnum], [plop get_apiname], [plop get_errmsg]];
            *error = [[NSError alloc] initWithDomain:errorText code:[plop get_errnum] userInfo:nil];
            return nil;
        }
        
        /* --------- general information (always available) */
        
        pcosmode = [plop pcos_get_number:doc path:@"pcosmode"];
        
        [buffer appendFormat:@"   File name: %@\n", [plop pcos_get_string:doc path:@"filename"]];
        
        [buffer appendFormat:@" PDF version: %@\n", [plop pcos_get_string:doc path:@"pdfversionstring"]];
        
        [buffer appendFormat:@"  Encryption: %@\n",
            [plop pcos_get_string:doc path:@"encrypt/description"]];
        
        recipientcount = [plop pcos_get_number:doc path:@"length:encrypt/recipients"];

        if (recipientcount > 0)
        {
            [buffer appendFormat:@"Recipient groups: %d\n", (int)recipientcount];
            [buffer appendFormat:@"      Digital ID: "];

            if (pcosmode == 2)
                [buffer appendFormat:@"supplied digital ID has master permissions\n"];
            else if (pcosmode == 1)
                [buffer appendFormat:@"supplied digital ID doesn't have master permissions\n"];
            else
                [buffer appendFormat:@"no suitable digital ID supplied\n"];
        }

        [buffer appendFormat:@"   Master pw: %@\n",
            [plop pcos_get_number:doc path:@"encrypt/master"] ? @"yes" : @"no"];
        
        [buffer appendFormat:@"     User pw: %@\n",
            [plop pcos_get_number:doc path:@"encrypt/user"] ? @"yes" : @"no"];
        
        [buffer appendFormat:@"Text copying: %@\n",
            [plop pcos_get_number:doc path:@"encrypt/nocopy"] ? @"yes" : @"no"];
        
        [buffer appendFormat:@"  Linearized: %@\n",
            [plop pcos_get_number:doc path:@"linearized"] ? @"yes" : @"no"];
        
        if (pcosmode == 0)
        {
            [buffer appendString:@"Minimum mode: no more information available\n\n"];
            [plop close_document:doc optlist:@""];
            return buffer;
        }
        
        /* --------- more details (requires at least user password) */
        
        [buffer appendFormat:@"PDF/X status: %@\n", [plop pcos_get_string:doc path:@"pdfx"]];
        
        [buffer appendFormat:@"PDF/A status: %@\n", [plop pcos_get_string:doc path:@"pdfa"]];

        [buffer appendFormat:@"PDF/UA status: %@\n", [plop pcos_get_string:doc path:@"pdfua"]];

        [buffer appendFormat:@"PDF/VT status: %@\n", [plop pcos_get_string:doc path:@"pdfvt"]];
        
        [buffer appendFormat:@"    XFA data: %@\n",
            (int) [plop pcos_get_number:doc path:@"type:/Root/AcroForm/XFA"]
            != pcos_ot_null ? @"yes" : @"no"];
        
        [buffer appendFormat:@"  Tagged PDF: %@\n\n",
            [plop pcos_get_number:doc path:@"tagged"] ? @"yes" : @"no"];
        
        /* --------- analyze digital signatures */

        count = [plop pcos_get_number:doc path:@"length:signaturefields"];

        for (i=0; i < count; i++)
        {
            NSString *sigtype;

            [buffer appendFormat:@"   Sig. field #%d: '%@',", (int)i+1,
                [plop pcos_get_string:doc path:[NSString
                stringWithFormat:@"signaturefields[%d]/fullname", (int)i]]];

            [buffer appendFormat:@" %@",
                [plop pcos_get_number:doc path:[NSString
                stringWithFormat:@"signaturefields[%d]/visible", (int)i]]
                ? @"visible" : @"invisible"];

            sigtype = [plop pcos_get_string:doc path:[NSString
                stringWithFormat:@"signaturefields[%d]/sigtype", (int)i]];

            if ([sigtype isEqualToString:@"none"])
            {
                [buffer appendFormat:@", unsigned\n"];
                continue;
            }

            [buffer appendFormat:@" %@ signature", sigtype];

            if ([sigtype isEqualToString:@"certification"])
            {
                [buffer appendFormat:@" (%@ allowed)", [plop pcos_get_string:doc
                        path:[NSString stringWithFormat:@"signaturefields[%d]/permissions", (int)i]]];
            }

            if ([plop pcos_get_number:doc path:[NSString
                 stringWithFormat:@"signaturefields[%d]/cades", (int)i]])
                [buffer appendFormat:@", CAdES"];

            [buffer appendFormat:@"\n"];
        }
        if (count > 0)
        {
            [buffer appendFormat:@"No. of revisions: %d\n\n",
                (int)[plop pcos_get_number:doc path:@"revisions"]];
        }

        [buffer appendFormat:@"  Reader-enabled: %@\n",
            [plop pcos_get_string:doc path:@"usagerights"]];

        [buffer appendFormat:@"No. of pages: %d\n",
         (int) [plop pcos_get_number:doc path: @"length:pages"]];
        
        [buffer appendFormat:@" Page 1 size: width=%g, height=%g\n",
         [plop pcos_get_number:doc path:[NSString stringWithFormat:@"pages[%d]/width", 0]],
         [plop pcos_get_number:doc path:[NSString stringWithFormat:@"pages[%d]/height", 0]]];
        
        count = (NSInteger) [plop pcos_get_number:doc path:@"length:fonts"];
        [buffer appendFormat:@"No. of fonts: %ld\n", (long)count];
        
        for (i=0; i < count; i++)
        {
            if ([plop pcos_get_number:doc path:[NSString stringWithFormat:@ "fonts[%ld]/embedded", (long)i]])
                [buffer appendString:@"embedded "];
            else
                [buffer appendString:@"unembedded "];
            
            [buffer appendFormat:@"%@ font ",
             [plop pcos_get_string:doc path:[NSString stringWithFormat:@"fonts[%ld]/type", (long)i]]];
            [buffer appendFormat:@"%@\n",
             [plop pcos_get_string:doc path:[NSString stringWithFormat:@"fonts[%ld]/name", (long)i]]];
        }
        
        [buffer appendString:@"\n"];
        
        plainmetadata = (NSInteger) [plop pcos_get_number:doc path:@"encrypt/plainmetadata"];
        
        if (pcosmode == 1 && !plainmetadata && ((NSInteger) [plop pcos_get_number:doc path:@"encrypt/nocopy"]))
        {
            [buffer appendString:@"Restricted mode: no more information available\n\n"];
            [plop close_document:doc optlist:@""];
            return buffer;
        }
        
        /* --------- document info keys and XMP metadata (requires master pw
         * or plaintext metadata)
         */
        
        count = (NSInteger) [plop pcos_get_number:doc path:@"length:/Info"];
        
        for (i=0; i < count; i++)
        {
            objtype = (NSInteger) [plop pcos_get_number:doc path:[NSString stringWithFormat:@"type:/Info[%ld]", (long)i]];
            
            [buffer appendFormat:@"%@: ", [plop pcos_get_string:doc path:[NSString stringWithFormat: @"/Info[%ld].key", (long)i]]];
            
            /* Info entries can be stored as string or name objects */
            if (objtype == pcos_ot_string || objtype == pcos_ot_name)
            {
                [buffer appendFormat:@"'%@'\n",
                 [plop pcos_get_string:doc path:[NSString stringWithFormat: @"/Info[%ld]", (long)i]]];
            }
            else
            {
                [buffer appendFormat:@"(%@ object)\n",
                 [plop pcos_get_string:doc path:[NSString stringWithFormat: @"type:/Info[%ld]", (long)i]]];
            }
        }
        
        [buffer appendFormat:@"\nXMP meta data: "];
        
        objtype = (NSInteger) [plop pcos_get_number:doc path:@"type:/Root/Metadata"];
        if (objtype == pcos_ot_stream)
        {
            NSData *contents;
            
            contents = [plop pcos_get_stream:doc optlist:@"" path:@"/Root/Metadata"];
            [buffer appendFormat:@"%lu bytes ", (unsigned long)contents.length];
            
            NSString *unicode = [[NSString alloc] initWithData:contents encoding:NSUTF8StringEncoding];
            [buffer appendFormat:@"(%lu Unicode characters)\n\n", (unsigned long)unicode.length/2];
        }
        else
        {
            [buffer appendString:@"not present\n\n"];
        }
        
        [plop close_document:doc optlist:@""];
    }    
    @catch (PLOPException *ex) {
        NSString *exception = [NSString stringWithFormat:@"Error %ld in %@(): %@\n",
                     (long)[ex get_errnum], [ex get_apiname], [ex get_errmsg]];
        *error = [[NSError alloc] initWithDomain:exception code:[ex get_errnum] userInfo:nil];
        return nil;
    }
    
    return buffer;
}

@end
