//
//  signViewController.m
//  PLOP examples
//
//  Created by Rainer Schaaf on 09.01.15.
//  Copyright (c) 2015 Rainer Schaaf. All rights reserved.
//

#import "sign.h"
#import "PLOP_ios/PLOP_objc.h"

@implementation sign

+ (NSData *) createPdf:(NSString *) fileName error:(NSError**)error {
    NSData *pdfData=nil;
    
    // Basic signature options
    NSString *sign_opts =
        @"ltv=try "         // use ltv=full to ensure LTV status */
        // for use with PKCS11
        // @"engine=pkcs#11 digitalid={filename=cryptoki.dll} "
        @"engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} ";

    bool visualize = false;    // Set to "true" to enable signature visualization
    NSString *visfile = [[NSBundle mainBundle] pathForResource:@"signing_man_pdfa1a" ofType:@"pdf"];
    
    PLOP *plop = [[PLOP alloc] init];
    if (!plop) {
        *error = [[NSError alloc] initWithDomain:@"PLOP out of memory error." code:[plop get_errnum] userInfo:nil];
        return nil;
    }
    
    @try {
        NSInteger doc;
        NSInteger visdoc = -1;
        NSString *optlist = sign_opts;

        
        // look for resources in "resourcePath"
        [plop set_option:[NSString stringWithFormat:@"searchpath {%@} ", [[NSBundle mainBundle] resourcePath]]];
        
        // open unprotected input file without any password
        if ((doc = [plop open_document:fileName optlist:@""]) == -1) {
            *error = [[NSError alloc] initWithDomain:[NSString stringWithFormat:@"Error: %@: %@", [plop get_apiname], [plop get_errmsg]] code:[plop get_errnum] userInfo:nil];
            return nil;
        }
        
        if (visualize){
            NSString *fieldoptlist;
            if ((visdoc = [plop open_document:visfile optlist:@""]) == -1) {
                *error = [[NSError alloc] initWithDomain:[NSString stringWithFormat:@"Error: %@: %@", [plop get_apiname], [plop get_errmsg]] code:[plop get_errnum] userInfo:nil];
                return nil;
            }
            fieldoptlist = [NSString stringWithFormat:@"field={visdoc=%ld rect={50 50 adapt adapt}} ", (long)visdoc];
            optlist = [NSString stringWithFormat:@"%@%@", optlist, fieldoptlist];
        }
        
        if ([plop prepare_signature:optlist] == -1) {
            *error = [[NSError alloc] initWithDomain:[NSString stringWithFormat:@"Error: %@: %@", [plop get_apiname], [plop get_errmsg]] code:[plop get_errnum] userInfo:nil];
            return nil;
        }
        
        if (([plop create_document:@"" optlist:[NSString stringWithFormat:@"input=%ld", (long)doc]]) == -1) {
            *error = [[NSError alloc] initWithDomain:[NSString stringWithFormat:@"Error: %@: %@", [plop get_apiname], [plop get_errmsg]] code:[plop get_errnum] userInfo:nil];
            return nil;
        }
        
        pdfData = [plop get_buffer];
        
        // close input and output files
        [plop close_document:doc optlist:@""];
        
        if (visualize){
            [plop close_document:visdoc optlist:@""];
        }
    }
    @catch (PLOPException *ex) {
        NSString *errorText = [NSString stringWithFormat:@"Error %ld in %@(): %@\n", (long)[ex get_errnum], [ex get_apiname], [ex get_errmsg]];
        *error = [[NSError alloc] initWithDomain:errorText code:[ex get_errnum] userInfo:nil];
        return nil;
    }
    
    return pdfData;
}

@end
