/* 
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 * mini sample for inserting XMP metadata in PDF documents
 *
 */

#include <stdio.h>

#include "ploplib.h"

int
main(void)
{
    PLOP *plop;

    /* Directory where files will be searched */
    const char *searchpath = "../data";

    /* input document */
    const char *in_filename = "PLOP-datasheet.pdf";

    /* output document */
    const char *out_filename = "PLOP-datasheet-xmp.pdf";

    /* Option list for the generated output document.
     * Some alternatives for the option list using the supplied XMP samples:
     *
     * Insert custom metadata for the GWG (Ghent Workgroup)
     * Ad Ticket scheme (see http://www.gwg.org/Jobtickets.phtml):
     * "metadata={filename=gwg_ad_ticket.xmp validate=xmp2004}"
     *
     * Insert custom metadata based on Acme's enterprise schema:
     * "metadata={filename=acme.xmp}"
     *
     * Insert a custom "engineering" schema for scanned documents:
     * This works best with a PDF/A-1 input document (e.g. mini_pdfa1.pdf),
     * and requires XMP input which conforms to PDF/A-1:
     * "metadata={filename=engineering.xmp}"
     *
     * Insert a PDF/A extension schema after validating the XMP for PDF/A-1.
     * This works best with a PDF/A-1 input document (e.g. mini_pdfa1.pdf),
     * and requires XMP input which conforms to PDF/A-1:
     * "metadata={filename=machine_pdfa1.xmp validate=pdfa1}"
     *
     */
    const char *base_opts =
        "metadata={filename=gwg_ad_ticket.xmp}";


    /* create a new PLOP object */
    if ((plop = PLOP_new()) == (PLOP *) 0)
    {
        printf("Couldn't create PLOP object (out of memory)!\n");
        return(2);
    }

    PLOP_TRY(plop) {
        int doc;
        char optlist[1024];

        sprintf(optlist, "searchpath={%s}", searchpath);
        PLOP_set_option(plop, optlist);

        /* open input file */
        if ((doc = PLOP_open_document(plop, in_filename, 0, "")) == -1)
        {
            printf("Error: %s\n", PLOP_get_errmsg(plop));
            PLOP_delete(plop);
            return(2);
        }

        /* create the output file and add XMP metadata */
        sprintf(optlist, "input=%d %s", doc, base_opts);
        if (PLOP_create_document(plop, out_filename, 0, optlist) == -1)
        {
            printf("Error: %s\n", PLOP_get_errmsg(plop));
            PLOP_delete(plop);
            return(2);
        }

        /* close input file */
        PLOP_close_document(plop, doc, "");
    }

    PLOP_CATCH(plop) {
        printf("PLOP exception occurred in insertxmp:\n");
        printf("[%d] %s: %s\n", PLOP_get_errnum(plop),
                PLOP_get_apiname(plop), PLOP_get_errmsg(plop));
        PLOP_delete(plop);
        return(2);
    }

    /* delete the PLOP object */
    PLOP_delete(plop);

    return(0);
}
